//~ Ping Genius Loci processing interaction engine
//~ Adam Somlai-Fischer 
//~ aether architecture

// @ todo
// for showing available power, sensor reading, etc

// The Obsessive Camera Direction
// find link at http://www.processing.org/reference/libraries/index.html
import damkjer.ocd.*;

// global vairables
int fieldWidth = 20;
int fieldLength = 20;
int pixelAmount = fieldWidth * fieldLength;
float pixelSpacing = 5.0;
float tempRotater = 0.0;

//~ float rotationSpeed = 255 / 30; // for 1 rotation / second, with 30 fps
float rotationSpeed = 140 / 30; // for 1 rotation / second, with 30 fps

// create all PingPixels
PixelGrid pixelGrid = new PixelGrid();

// set up views
Camera cameraTop;
Camera cameraBird;
Camera cameraHuman;
int selectedView = 1;
int selectedContent = 2;

void setup() {
	size(800, 600, P3D); 
	colorMode(HSB, 255); 
	framerate(30); 
	noStroke();
	// set up views
	cameraTop = new Camera(this, (fieldWidth * pixelSpacing) / 2, 
							-3000, 
							(fieldLength * pixelSpacing) / 2, 
							(fieldWidth * pixelSpacing) / 2, 
							0, 
							(fieldLength * pixelSpacing) / 2);
	cameraTop.zoom(radians(-57.5));
	
	cameraBird = new Camera(this, (fieldWidth * pixelSpacing), 
							-50, 
							(fieldLength * pixelSpacing), 
							(fieldWidth * pixelSpacing) / 2, 
							0, 
							(fieldLength * pixelSpacing) / 2);
	
	cameraHuman = new Camera(this, (fieldWidth * pixelSpacing), 
							-10, 
							(fieldLength * pixelSpacing), 
							(fieldWidth * pixelSpacing) / 2, 
							0, 
							(fieldLength * pixelSpacing) / 2);
} 
 

void draw() { 
	background(37,31,142);  // color of the shaded wooden stage
	lights();
	drawPixels();
	switch (selectedContent){
		case 1:
			displayPerlinNoise();
			break;
		case 2:
			displayWaves();
			break;
		case 3	:
			displayID();
			break;
		case 4	:
			displayRadiatingman();
			break;
	}
	switch (selectedView){
		case 1:
			cameraTop.feed();
			break;
		case 2:
			cameraBird.feed();
			cameraBirdNavigation();
			break;
		case 3:
			cameraHuman.feed();
			cameraHumanNavigation();
			break;
	}
} 

class PixelGrid {
	
	// array of PingPixels to contain all PingPixels
	PingPixel[] pingPixels = new PingPixel[pixelAmount]; 

	// create all PingPixels upon initilization	
	PixelGrid(){
		//~ int gLength = gWidth * gHeigth;
		int row = 0;
		int x = 0;
		int y = 0;
		
		for (int id = 0; id < pixelAmount; id++) {
			pingPixels[id] = new PingPixel(id); 
			if (row >= fieldWidth) {
				row = 0;
				x = 0;
				y += pixelSpacing;
			} 
			pingPixels[id].x = x;
			pingPixels[id].y = y;
			pingPixels[id].sensor = 0;
			row++;
			x += pixelSpacing;

		}
	}

	// function for returning PingPixel at x-y	
	PingPixel xy (int x, int y){
		int id = x + (y * fieldWidth);
		return pingPixels[id];
	}
	
	// function for returning PingPixel at ID	
	PingPixel id (int id){
		return pingPixels[id];
	}
	
	// wrap around 
	// when you step outside in one axis it enters
	// on the other side, wraps coordinates in x and y
	PingPixel xy_wrap(int x, int y){
		return xy(x%fieldWidth,y%fieldLength);
	}
	
	// ping
	// ping pong in both axis so if you step outside it bounces backs,
	// mirrors coordinate system in x and y and wraps
	PingPixel xy_pingPong(int x, int y){
		return xy(abs(x%(fieldWidth*2)-fieldWidth),abs(y%(fieldLength*2)-fieldLength));
	}
	
}


class PingPixel { 
	//~ int pixelID;
	int angle, sensor; 
	float speed, speaker, temperature, sunligth, power; 
	int x, y;
	
	PingPixel(int pixelID){
		angle	= 0;
		sensor = 0;
	}
	
	void rotateMe(int targetAngle){
		if (targetAngle < angle){
			angle -= rotationSpeed;
		}
		if (targetAngle > angle){
			angle += rotationSpeed;
		}
	}
} 

void randomizeSensors(){
	for (int id = 0; id < pixelAmount; id++) {
		pixelGrid.id(id).sensor = int(random(255));
	}	
}


// visualization ********************************************************

void drawPixels(){
	for (int id=0; id<pixelAmount; id++) {

		float halfBoxHeight = 1.9;
		float halfBoxWidth = 0.5;
			  	
		for (int j = 0;  j < 3;  j++){
			fill(1 + (85 * j), 255, 255);		
			pushMatrix();
				translate(pixelGrid.id(id).x, 0, pixelGrid.id(id).y);
				rotateY(radians(pixelGrid.id(id).angle) + (radians(120) * j));
				rotateX(radians(9.2));
				beginShape(QUADS);
					vertex(-halfBoxWidth, halfBoxHeight, 0);
					vertex(halfBoxWidth, halfBoxHeight, 0);
					vertex(halfBoxWidth, -halfBoxHeight, 0);
					vertex(-halfBoxWidth, -halfBoxHeight, 0);
					vertex(-halfBoxWidth, halfBoxHeight, 0);
				endShape(); 
				// white angle vector, only on top view
				if (j == 0 && selectedView == 1){
					stroke(0,0,255);
					line(0,0,0,2,0,0);
					noStroke();
				}

					
			popMatrix();
		}
		if (selectedContent == 4){
			pushMatrix();
				translate(pixelGrid.id(id).x, 0, pixelGrid.id(id).y);
				noFill();
				stroke(1);
				line(0,0,0,0,-pixelGrid.id(id).sensor / 20,0);
				translate(0,-pixelGrid.id(id).sensor / 20,0);
				box(pixelGrid.id(id).sensor / 150);
				noStroke();
			popMatrix();
		}
	}
	if (selectedContent == 4 && true){
		noFill();
		stroke(1);
		tempRotater += 0.03;
		//int manX = int((frameCount%fieldWidth) * pixelSpacing);
		int manX = int((sin(tempRotater) * fieldWidth/2 + fieldWidth / 2 + 5) * pixelSpacing);
		//int manY = int((frameCount%fieldLength) * pixelSpacing);
		int manY = int((cos(tempRotater) * fieldLength/2 + fieldLength / 2 + 5) * pixelSpacing);
		translate(manX, -2, manY);
		box(3,8,3);
		line(0,0,0,0,-20,0);
		translate(0,-20,0);
		box(1,1,1);
		noStroke();
		for(int i=0; i<5; i++) {
			for(int j=0; j<5; j++) {
				int a = min(int(manX/pixelSpacing) + i, fieldWidth-1);
				int b = min(int(manY/pixelSpacing) + j, fieldLength-1);
				pixelGrid.xy(a, b).rotateMe(40);
			}
		}
	}
}


// animated content

// perlin noise, natural looking fields
void displayPerlinNoise(){
	float noiseScale=0.02; 	// this sets scale of pattern, higher is more dense
	//~ float noiseDepth =  millis() * 0.003; 	// this sets speed of change
	int noiseDepth =  second() * 10;	// this makes clock like behaviour
	for(int i=0; i<fieldWidth; i++) {
		for(int j=0; j<fieldLength; j++) {
			pixelGrid.xy(i,j).rotateMe(int(noise(i*noiseScale, j*noiseScale, noiseDepth*noiseScale)*1000));
		}
	}
}

// circular waves pulling us in...
void displayWaves(){
	float waveScale=1800; 	// this sets scale of pattern, higher is more dense
	float waveDepth =  millis() * 0.0002;	// waveing speed
	for(int i=0; i<fieldWidth; i++) {
		for(int j=0; j<fieldLength; j++) {
			pixelGrid.xy(i,j).rotateMe(int(sin(waveDepth + 
			radians(sqrt(pow(((fieldWidth / 2) - i), 2) + pow(((fieldLength / 2) - j), 2)))) * waveScale));
		}
	}
}

// for checking if placement is OK
void displayID(){
	float waveScale=500; 	// this sets scale of pattern, higher is more dense
	float waveDepth =  millis() * 0.02;	// waveing speed
	for(int i=0; i<fieldWidth; i++) {
		for(int j=0; j<fieldLength; j++) {
			pixelGrid.xy(i,j).rotateMe(i*5);
		}
	}
}

// interactive content
// circular waves pulling us in... 
void displayRadiatingman(){
	float waveScale=1800; 	// this sets scale of pattern, higher is more dense
	float waveDepth =  millis() * 0.0002;	// waveing speed
	int manX = int(random(fieldWidth));
	int manY = int(random(fieldLength));
	
	for(int i=0; i<fieldWidth; i++) {
		for(int j=0; j<fieldLength; j++) {
			pixelGrid.xy(i,j).rotateMe(int(sin(waveDepth + 
			radians(sqrt(pow(((fieldWidth / 2) - i), 2) + pow(((fieldLength / 2) - j), 2)))) * waveScale));
			//pixelGrid.xy(i,j).rotateMe(i*5);
		}
	}
}

// user input
 	

void keyReleased() { 
	// switch through cameras
	if (key == '1'){
		selectedView = 1;
	} else if (key == '2'){
		selectedView = 2;
	} else if (key == '3'){
		selectedView = 3;
	}
	
	// witch through content
	if (key == 'a'){
		selectedContent = 1;
	} else if (key == 's'){
		selectedContent = 2;
	} else if (key == 'd'){
		selectedContent = 3;
	} else if (key == 'f'){
		selectedContent = 4;
	}
	
	// unique actions
	if (key == 'r'){
		randomizeSensors();
	}
}

// camera navigation
void cameraBirdNavigation(){
	cameraBird.tumble(radians(mouseX - pmouseX), radians(mouseY - pmouseY));
}


void cameraHumanNavigation(){
	cameraHuman.look(radians(mouseX - pmouseX) / 16.0, radians(mouseY - pmouseY) / 12.0);
	if(mousePressed == true && mouseButton == LEFT){
		cameraHuman.dolly(-0.2);
	} else if(mousePressed == true && mouseButton == RIGHT){
		cameraHuman.dolly(0.2);
	}  
}
